;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x0dba8a488948199c22556a823e6d2111 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x552003252b13e103b708fa264f068d88 . "./code.cg2js/cg-printing.fasl")
(#x91aed65505d528b97dee14353c457454 . "./code.cg2js/cg-caret.fasl")
(#x3d1458d8f93613974aa77321ac726cc8 . "./code.cg2js/cg-item-list.fasl")
(#xc0479eb12009ffdc985f7c9e11691925 . "./code.cg2js/cg-progress-indicator.fasl")
(#xacef77fdf62d5b14ac147cd1abbee697 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xcdc7c4d5502f0a686426c50a75ab3845 . "./code.cg2js/cg-outline.fasl")
(#xa6a9b8e4f697d18ff1dcd955c8765f81 . "./code.cg2js/cg-palette.fasl")
(#x1f10b76590d3a8084473c971678419f8 . "./code.cg2js/cg-selected-object.fasl")
(#x273807721c9cfd45d872b191593b39fa . "./code.cg2js/cg-status-bar.fasl")
(#xa20e36dfc5b80b3fbe3613cd156bded1 . "./code.cg2js/cg-wrap-string.fasl")
(#x8f82f1a8b7f98269e808025f316a5dee . "./code.cg2js/cg-fill-texture.fasl")
(#x82dbaaaac4bca70d6f02a5dc2aff9927 . "./code.cg2js/cg-layout.fasl")
(#x7e1948c23d69b5d83a612a443a5d4edf . "./code.cg2js/cg-timer.fasl")
(#x429d3b22f018aa79811189746caad267 . "./code.cg2js/cg-dialog-item.fasl")
(#x952734cdf04b9f1efb65465b74be87bd . "./code.cg2js/cg-button.fasl")
(#x5923a2598ba5c7c75fa8e51b326d2a20 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x85ccb2f7931696567bd14a43370237d9 . "./code.cg2js/cg-tray.fasl")
(#xd0b4b8b7038fbafe3d433b43f42a1832 . "./code.cg2js/cg-plot-widget.fasl")
(#xc93833185873774fa1ed80a258443a68 . "./code.cg2js/cg-pixmap.fasl")
(#x0f792ca5cfed1e9f3427136ce6af299e . "./code.cg2js/cg-font-dialog.fasl")
(#xefb29fd3ed4b05b3e7f2c01a01bd4cbb . "./code.cg2js/cg-os-widget.fasl")
(#xf77a41a3403466419b1d63da1295e394 . "./code.cg2js/cg-text-or-combo.fasl")
(#xbeb40a9769fba2a1ee9d2006cd72094b . "./code.cg2js/cg-trackbar.fasl")
(#x29e615b6515f48aed3cf35420d889950 . "./code.cg2js/cg-up-down-control.fasl")
(#x7b3dc429374d59a0320d36d3d78433be . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xbcc251a0fcf4db679a491fef8520f40d . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x53a3836ac3f6b97049f17bf2c8430411 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x7db9a95a839bae990a362a8e83901b9e . "./code.cg2js/cg-bitmap-stream.fasl")
(#x37bade6fd47070beedda803d60f25f6d . "./code.cg2js/cg-chart-widget.fasl")
(#x4e77af27407184023058d2866b0ec7b8 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd29eb602c35bcb5edb089d7e83845775 . "./code.cg2js/cg-group-box.fasl")
(#x9918687f2b8b4ea0da3a41a404aa4610 . "./code.cg2js/cg-static-text.fasl")
(#xefd33ea6ecc9bb40388e22b0775fdf5a . "./code.cg2js/cg-class-slot-grid.fasl")
(#xbf11b07f1e7e38649ff5aef81f6412a4 . "./code.cg2js/cg-color-dialog.fasl")
(#xb5cf18261a2ed0a2223cbf26154b7a95 . "./code.cg2js/cg-intersect.fasl")
(#xe699b9635c28b1e94c520fa138a75e00 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1692946f9a79dff642c52ee2c7b426ef . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x1b0a19f918e0df3be1c99d0f7e30d506 . "./code.cg2js/cg-html-dialog.fasl")
(#x5db426553e46b647c2279a500a2c7553 . "./code.cg2js/cg-lettered-menu.fasl")
(#x214fe900999c9917d39ebf1d274b1ab7 . "./code.cg2js/cg-class-grid.fasl")
(#x3ab2f2b25b9527ac8827d7255477ef83 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x512fa8d48e606bf3a1d50d4a3852b05a . "./code.cg2js/cg-calendar.fasl")
(#x47646f5d06a712416138f2e2345f16cb . "./code.cg2js/cg-editable-text.fasl")
(#xfd54a8fdfd96abf2dbdc800f6dae7820 . "./code.cg2js/cg-property.fasl")
(#xe9adec2c1e47bcd610b1231d79bcc40e . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xcb4b5a629f063db22d5332904c806a2b . "./code.cg2js/cg-grid-widget.fasl")
(#x34979b1986f8361e434c52d78068a9af . "./code.cg2js/cg-paren-matching.fasl")
(#x30280c74977fb1988310f92ee70a005d . "./code.cg2js/cg-dev.fasl")
(#x918055f5c3a0bb58707ae6cdfee16594 . "./code.cg2js/cg-drawable.fasl")
(#xa5e0c07351d996d84a3644f62ecdbda7 . "./code.cg2js/cg-curve.fasl")
(#x880f017cca7ace5d1574ce7d3e3e7c58 . "./code.cg2js/cg-lisp-widget.fasl")
(#xb0d91de9c77cfffebde9eb70b34c9687 . "./code.cg2js/cg-dde-utils.fasl")
(#x2a6c55d42ce934283ea999d69e7fe967 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x980383ec781fbaaf0d44d7ba166d8d79 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xc2743cba54611db0ffc24776d9d946ce . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xc61e793b792f25d145fb1cfd0cf0e3e5 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x15bce0bdf04c032e4d8eec2a21f0059c . "./code.cg2js/cg-lamp.fasl")
(#x628bf466ac4636853a37f0418eab491c . "./code.cg2js/cg-common-control.fasl")
(#xc8d849aff1d2722210900ca493818bf7 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x45fa988d4937daef626020abd62fd6a7 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x194332c48e29a7117901fe4d51d4086f . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xea88f523d90a6deff9f0459133990b02 . "./code.cg2js/cg-project-window.fasl")
(#x5dbde98aea0638bc8191323330208a8f . "./code.cg2js/cg-sample-file-menu.fasl")
(#x5e38d05269a3d86d5d2ac26d46f67043 . "./code.cg2js/cg-class-support.fasl")
(#xd8f87f44c4c965eb14c8feb222903963 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xfc3958b0e261f47b99f97ff0a1f9d10b . "./code.cg2js/cg-yes-no-list.fasl")
(#xe9562956246410128d417f6949b66177 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x2b27fdccf488b94e5698b2c9c065c5e1 . "./code.cg2js/cg-string-dialog.fasl")
(#xf97e010c3611642164d53382787bb725 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x9c72d82bcf5f3ee488202e007b4d418f . "./code.cg2js/cg-dropping-outline.fasl")
(#x1624e50cecaada4a62a7566be82c8fb2 . "./code.cg2js/cg.fasl")
(#x4357d2baae95bfad683cafa919d1d424 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x521c1c60f2f1a9744cf0548e10f366a4 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3db96b079ee6f595ed8f37a639526ec3 . "./code.cg2js/cg-yes-no-string.fasl")
(#xff08f6151b3a55ec0c29be2c9b6fdeda . "./code.cg2js/cg-icon-pixmap.fasl")
(#x0d1d996d09265ac00a07b719f2361a38 . "./code.cg2js/cg-web-browser.fasl")
(#x3a163f0e029531d8315565bcca652d0d . "./code.cg2js/cg-clipboard-stack.fasl")
(#x1992f2773b1db55f07b3cd4d2e65b96c . "./code.cg2js/cg-drag-and-drop.fasl")
(#x684303d36af45d1e7075ff1574a7b91f . "./code.cg2js/cg-message-dialog.fasl")
(#x00ad0b3cdd0ef77f5a247f0c18214b80 . "./code.cg2js/cg-comtab.fasl")
(#x251018c286a2d62d0be74ea38fd16179 . "./code.cg2js/cg-base.fasl")
(#x9ce31e428315e0a7c2669eaea210a6e9 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x7e3bf112e91ecbf643271d3812fcb013 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x90c7f021331c4c85470237bdee4192db . "./code.cg2js/cg-utility-dialog.fasl")
(#x00b17a38ff0bf961b3839b18a517d193 . "./code.cg2js/cg-object-editor.fasl")
(#xd5c36f72d480e6fd0d6901fa3a007c51 . "./code.cg2js/cg-edit-in-place.fasl")
(#xd3e6d5b836a8a6862a9e5e0cb2e11803 . "./code.cg2js/cg-radio-button.fasl")
(#xc0029eabe9923dba2446c90fb7d1e286 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x97e8b29e23e37c8053a47a132dd1d9a9 . "./code.cg2js/cg-nodes.fasl")
(#x14181bb4e3d8a517ad2a2aea8c16371a . "./code.cg2js/cg-get-position.fasl")
(#xa231eee1b2a58575ba2e9f4f199a9d3e . "./code.cg2js/cg-directory-dialog.fasl")
(#xc61b3023355b6b7222dff9da41966426 . "./code.cg2js/cg-lisp-text.fasl")
(#xb23c6f27facaf4ba0759e54c318b2579 . "./code.cg2js/cg-combo-box.fasl")
(#x200b367c47b37c95eb65232086af4b93 . "./code.cg2js/cg-text-widget.fasl")
(#xb29fea7f2ec0999298207940124cbba5 . "./code.cg2js/cg-list-view.fasl")
(#x2d5f7d5f0ac080eccbf18a8a4f8984ee . "./code.cg2js/cg-graphics-context.fasl")
(#x47469e70d99515c3d09a2c93d6d432ea . "./code.cg2js/cg-get-pixmap.fasl")
(#x0caf15a6036dbd32aa8a0e58423a7cab . "./code.cg2js/cg-menu.fasl")
(#xf8d034e6c60969b642e721d4b3f59f99 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xb95562a65af8ba4a64ed64863b5dca25 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xda8470395c062a70a3d26cdeed9a94d6 . "./code.cg2js/cg-tooltip.fasl")
(#xb715b23c6d6b121ee0df239df912ce4b . "./code.cg2js/cg-html-widget.fasl")
(#x2baec9bcccda694d36297aa8585b6fc2 . "./code.cg2js/cg-choice-list.fasl")
(#xe1fafcd42df0ae01563942a4d9cc194f . "./code.cg2js/cg-icon.fasl")
(#xb7d9a7091bc234e9fe30a44b04d486e7 . "./code.cg2js/cg-acache.fasl")
(#xce0909fea27ef1dd30675bfc72d9c9ae . "./code.cg2js/cg-template-string.fasl")
(#x9bc9ecfab4066c5a9b0a3a8f7f722f4c . "./code.cg2js/cg-bitmap-pane.fasl")
(#x4ab9f57e20837b3753160ebd0408b7d9 . "./code.cg2js/cg-toggling-widget.fasl")
(#x4c8e023b372bd57d798234e99aea2a22 . "./code.cg2js/cg-tab-control.fasl")
(#xe3d8a665e44280a55e72f00ffd5b7531 . "./code.cg2js/cg-split-bar.fasl")
(#xbbc0fd22b53abb3a8ca92355e0e10a3f . "./code.cg2js/cg-picture-widget.fasl")
(#x3f31c882ac1739ad17a48471b7b2493f . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xa0ce27051213386b2518e3ec48031dc1 . "./code.cg2js/cg-choose-printer.fasl")
(#x24cb91005fee9380d63bfd95f941cdfb . "./code.cg2js/cg-os-window.fasl")
(#xc56d1d66985c68be57342eced822bd6f . "./code.cg2js/cg-scroll-bar.fasl")
(#xe2f4e53df4a7d0e57e1b0ab75d48d614 . "./code.cg2js/cg-clipboard.fasl")
(#xa887213d5f4ae5c95ac718cf169f19af . "./code.cg2js/cg-find-string-dialog.fasl")
(#x09fa8b60a3a0ba01a74ef0c05ac52a91 . "./code.cg2js/cg-hotspot.fasl")
(#x7ac8810c93b4840a6aec74f46f98d88a . "./code.cg2js/cg-check-box.fasl")
(#xe698039f2c3e88d7834ab367b2b94dd3 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x0482b41ef2ef0f16d5e0a7184e7867d9 . "./code.cg2js/cg-file-dialog.fasl")
(#xeb133e8f60b294558054168747f01292 . "./code.cg2js/cg-toolbar.fasl")
(#xbf5b1a3f5fb9f2fd818a236ea5c54e54 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x2bfc1be4d22c766945e32aac89f7d314 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x74e42f007baa27bf80c0a80714376522 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x083240bf19509e6687f8ffcbd9ea7797 . "./code.idejs/ide-project.fasl")
(#x2d10c10b4ad580320038ae2db8708c6e . "./code.idejs/ide-intro.fasl")
(#x43cafc8f2d31ea24f22bbb2cfd0d6dd3 . "./code.idejs/ide-find-in-files.fasl")
(#xea620df00b37fa8409baadd47fe871a5 . "./code.idejs/ide-grapher.fasl")
(#x9a51e90fc225f1b4c14fe33bb5f3013b . "./code.idejs/ide-inspector.fasl")
(#xb0811fb2f38e7e81f28dae111aa8d447 . "./code.idejs/ide-editor.fasl")
(#x300428c5806b9aa3a9ccf6572302b9fb . "./code.idejs/ide-window-list.fasl")
(#x4bdc7cb40d08c0af41a26d8ddfcfce0b . "./code.idejs/ide-process-browser.fasl")
(#x1f25c05a1f01a32af58d2252f5c85a33 . "./code.idejs/ide-patch-dialog.fasl")
(#x152d087dc7d610507a29f6f3cd74657e . "./code.idejs/ide-class-grapher.fasl")
(#xca74c1819cf9283d30bb3a4e2c92efcc . "./code.idejs/ide-base.fasl")
(#x370038aec9035bdb283182330c5c4835 . "./code.idejs/ide.fasl")
(#xc05bebfc6dc68193ddb611f5d9ce5a24 . "./code.idejs/ide-trace.fasl")
(#x6ad69406657caa6b2842ed6f21eb81de . "./code.idejs/ide-form.fasl")
(#xa9dbb26df6d7a97f0a6a077c160d69d6 . "./code.idejs/ide-find-definitions.fasl")
(#x517146efa0a68db54e64ff4ad7f41e6a . "./code.idejs/ide-apropos.fasl")
(#x8544798bc72b2e62d751a5e1a89b5479 . "./code.idejs/ide-class-browser.fasl")
(#xce92e3a99dc042bdcc08459459f600d2 . "./code.idejs/ide-navigator.fasl")
(#xcd3890b6c5da7a87c2b090d13cf9db08 . "./code.idejs/ide-clipboard.fasl")
(#x085a66168bfe20e1ea835dc9c7779e0e . "./code.idejs/ide-acache.fasl")
(#x34c8f8babdebbacfd313e1992300c6e3 . "./code.idejs/ide-keyshort.fasl")
(#xc09f02d63171abbe344510bf0fe2319b . "./code.idejs/ide-package-list.fasl")
(#xd1c36e9bdfbde4059fece3ca2371940f . "./code.idejs/ide-dev.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
